"""
All commands use bsdtar and accept a logmethod to use for logging.
TODO: Make an iso class that can return things like arch, series, type, etc.
TODO: Add the existing functions to the iso class.
TODO: Convert the code to use the class instead of the existing functions.
"""
import os
import subprocess

from utah.exceptions import UTAHException


def listfiles(image, logmethod=None, returnlist=False):
    """
    Return either a subprocess instance listing the contents of an iso, or a
    list of files in the iso if returnlist is True.
    """
    cmd = ['bsdtar', '-t', '-f', image]
    if logmethod is not None:
        logmethod('bsdtar list command: ' + ' '.join(cmd))
    if returnlist:
        return subprocess.check_output(cmd).strip().split('\n')
    else:
        return subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE).communicate()


def getrealfile(image, filename, outdir=None, logmethod=None):
    """
    Return a command to safely extract a file from an iso.
    Based on unbsdtar-safelink from ubuntu iso testing.
    """
    cmd = ['bsdtar', '-t', '-v', '-f', image, filename]
    if logmethod is not None:
        logmethod('bsdtar list command: ' + ' '.join(cmd))
    output = (subprocess.Popen(cmd, stdout=subprocess.PIPE)
              .communicate()[0].split('\n')[0].split())
    try:
        if output[0].startswith('l'):
            if outdir is None:
                cmd = ['ln', '-s', output[-1], output[-3]]
            else:
                cmd = ['ln', '-s', output[-1],
                       os.path.join(outdir, output[-3])]
        else:
            realfile = output[-1]
            if outdir is None:
                cmd = ['bsdtar', '-x', '-f', image, '-O', realfile]
            else:
                cmd = ['bsdtar', '-x', '-f', image, '-C', outdir,
                       '-O', realfile]
    except IndexError:
        raise UTAHException('Cannot unpack image: ' + image)
    if logmethod is not None:
        logmethod('bsdtar extract command: ' + ' '.join(cmd))
    return cmd


def extract(image, filename, outdir=None, outfile=None, logmethod=None, **kw):
    """
    Return a subprocess to extract a file from an iso, passing any additional
    keyword arguments to the subprocess.
    If outfile is None, extract to standard out.
    If outdir is present and outfile is None, extract the file to outdir with
    the same name.
    If outdir and outfile are both present, extract to outfile in outdir.
    If the file is a hardlink, extract the actual file.
    If the file is a symlink, return a command to create the symlink.
    """
    if outfile is None:
        if outdir is None:
            cmd = getrealfile(image, filename, logmethod=logmethod)
            # The original function was designed to send the file over stdout,
            # this removes that
            # I think the updated function takes care of this already
            # if '-C' in cmd:
            #     cmd.remove('-C')
            if cmd[-1] != filename:
                if logmethod is not None:
                    logmethod(filename + ' appears to be a link to ' + cmd[-1])
                dirname = os.path.dirname(filename)
                if not os.path.isdir(dirname):
                    os.makedirs(dirname)
                os.chmod(dirname, 0775)
                if os.path.isfile(filename):
                    os.chmod(filename, 0775)
                return subprocess.call(cmd, stdout=open(filename, 'w'), **kw)
        else:
            cmd = getrealfile(image, filename,
                              outdir=outdir, logmethod=logmethod)
        if '-O' in cmd:
            cmd.remove('-O')
        return subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE, **kw).communicate()
    else:
        cmd = getrealfile(image, filename, logmethod=logmethod)
        return subprocess.call(cmd, stdout=open(outfile, 'w'), **kw)


def dump(image, filename, logmethod=None, **kw):
    """
    Return a subprocess dumping a file in an iso to standard out, passing any
    additional keyword arguments to the subprocess.
    """
    cmd = getrealfile(image, filename, logmethod=logmethod)
    return subprocess.Popen(cmd, stdout=subprocess.PIPE, **kw)
