#!/usr/bin/python

import os
import shutil
import tempfile
import time

import utah
from utah import config
from utah.provisioning.provisioning import (
        CustomInstallMixin,
        Machine,
        SSHMixin,
        )
from utah.provisioning.exceptions import UTAHProvisioningException
from utah.retry import retry


class CobblerMachine(CustomInstallMixin, SSHMixin, Machine):
    """
    Provide a class to provision a machine via cobbler.

    """
    # TODO: may need to fix DHCP/hostname issues, may just be magners
    def __init__(self, boot=None, cargs=None, inventory=None,
                 name=None, preseed=None, *args, **kw):
        if name is None:
            raise UTAHProvisioningException(
                    'Machine name reqired for cobbler machine')
        if cargs is None:
            raise UTAHProvisioningException(
                    'No cobbler arguments given for machine creation')
        else:
            self.cargs = cargs
        self.inventory = inventory
        if preseed is None:
            preseed = '/etc/utah/default-preseed.cfg'
        super(CobblerMachine, self).__init__(*args, name=name,
                                             preseed=preseed, **kw)
        if self.image is None:
            raise UTAHProvisioningException(
                    'Image file required for cobbler installation')
        self._custominit(arch=self.arch, boot=boot,
                         installtype=self.installtype, series=self.series)
        # TODO: support mini,
        # then desktop once we can figure out how
        self.cinitrd = None
        if self.installtype in ['alternate', 'server']:
            self.cinitrd = os.path.join('install', 'netboot',
                                        'ubuntu-installer', self.arch,
                                        'initrd.gz')
        else:
            raise UTAHProvisioningException(
                    'Only alternate and server images currently supported '
                    'for cobbler provisioning')
        if self.arch == 'amd64':
            self.carch = 'x86_64'
        else:
            self.carch = self.arch
        self.cname = self.name + '-' + self.carch
        self.logger.debug('Cobbler arch is ' + self.carch)
        self.logger.debug('Cobbler machine init finished')

    def _provision(self, checktimeout=config.checktimeout,
                   installtimeout=config.installtimeout):
        self.tmpdir = tempfile.mkdtemp(prefix='/tmp/' + self.name + '_')
        os.chdir(self.tmpdir)

        # TODO: try to remove this step, mount the iso and import that
        # with a specified kernel/preseed/initrd from the tmpdir
        self.logger.info('Extracting image to ' + self.tmpdir)
        files = utah.iso.listfiles(self.image, logmethod=self.logger.debug,
                                   returnlist=True)
        for myfile in files:
            utah.iso.extract(self.image, myfile, logmethod=self.logger.debug)

        self._preparekernel()
        cinitrd = os.path.join(self.tmpdir, self.cinitrd)
        initrd = self._prepareinitrd(initrd=cinitrd)
        self._unpackinitrd(initrd=initrd)
        self._setuplatecommand()
        self._setuppreseed()
        initrd = self._repackinitrd()

        os.chmod(cinitrd, 0755)
        shutil.copyfile(initrd, cinitrd)

        self.logger.info('Setting up system with cobbler')
        self.logger.debug('Removing old system')
        self._cobble(['system', 'remove', '--name=' + self.name])
        self.logger.info('Removing old profile')
        self._cobble(['profile', 'remove', '--name=' + self.cname])
        self.logger.info('Removing old distro')
        self._cobble(['distro', 'remove', '--name=' + self.cname])
        # TODO: support more image types, maybe do this without unpacking ISO
        self.logger.info('Importing image')
        self._cobble(['import', '--name=' + self.cname,
                      '--path=' + self.tmpdir, '--arch=' + self.carch])
        # TODO: figure out if I need kopts in both places
        self.logger.info('Adding kernel boot options to distro')
        self._cobble(['distro', 'edit', '--name=' + self.cname,
                      '--kopts=' + self.cmdline])
        self.logger.info('Adding kernel boot options '
                          'and kickstart to profile')
        preseed = os.path.join(self.tmpdir, 'initrd.d', 'preseed.cfg')
        self._cobble(['profile', 'edit', '--name=' + self.cname,
                      '--kickstart=' + preseed,
                      '--kopts=' + self.cmdline])
        self.logger.info('Adding system')
        self._cobble(['system', 'add', '--name=' + self.name,
                      '--profile=' + self.cname, '--netboot-enabled=Y']
                      + ['--' + arg + '=' + self.cargs[arg]
                         for arg in self.cargs])

        self.logger.info('Syncing cobbler')
        self._cobble(['sync'])
        self.restart()

        self.logger.info('Waiting for installation to begin')
        time.sleep(config.boottimeout)
        self._cobble(['system', 'edit', '--name=' + self.name,
                      '--netboot-enabled=N'])

        if self.debug:
            self.logger.info('Leaving temp directory '
                             'because debug is enabled: ' + self.tmpdir)
        else:
            self.logger.info('Cleaning up temp directory')
            # Cribbed from http://svn.python.org
            # /projects/python/trunk/Mac/BuildScript/build-installer.py
            for dirpath, dirnames, filenames in os.walk(self.tmpdir):
                for name in (dirnames + filenames):
                    filename = os.path.join(dirpath, name)
                    if not os.path.islink(filename):
                        self.logger.debug('Changing permissions of '
                                          + filename)
                        os.chmod(filename, 0775)
            shutil.rmtree(self.tmpdir)

        self.logger.info('System installing')
        self.logger.info('Checking every ' + str(checktimeout) +
                         ' seconds until system is installed')
        self.logger.info('Will time out after ' + str(installtimeout)
                         + ' seconds')
        retry(self.sshcheck, checktimeout, logmethod=self.logger.info)
        self.provisioned = True
        self.active = True
        uuid_check_command = ('[ "{uuid}" == "$(cat /etc/utah/uuid)" ]'
                              .format(uuid=self.uuid))
        if self.run(uuid_check_command)[0] != 0:
            self.provisioned = False
            raise UTAHProvisioningException(
                    'Installed UUID differs from CobblerMachine UUID; '
                    'installation probably failed. '
                    'Try restarting cobbler or running cobbler sync')
        self.logger.info('System installed')
        return True

    def destroy(self, *args, **kw):
        super(CobblerMachine, self).destroy(*args, **kw)
        self.__del__()
        del self

    def _cobble(self, cmd):
        """
        Pull out all cobbler commands so that we can later support changing
        users, remote login, not sudo, etc.
        """
        self._runargs(['sudo', 'cobbler'] + cmd)

    def _start(self):
        """
        Use cobbler to start the machine.
        """
        self.logger.debug('Starting system')
        self._cobble(['system', 'poweron', '--name=' + self.name])
        self.active = True

    def stop(self):
        """
        Use cobbler to stop the machine.
        """
        self.logger.debug('Stopping system')
        self._cobble(['system', 'poweroff', '--name=' + self.name])
        self.active = False

    def restart(self):
        self.logger.info('Restarting system')
        self.stop()
        self.logger.debug('Waiting ' + str(config.powertimeout) + ' seconds')
        time.sleep(config.powertimeout)
        self._start()

    def __del__(self):
        if self.inventory is not None:
            self.inventory.release(machine=self)

    def activecheck(self):
        """
        Start the machine if needed, and check for SSH login.
        """
        self.logger.debug('Checking if machine is active')
        self.provisioncheck()
        if not self.active:
            self._start()
            self.sshcheck()
