#!/usr/bin/python

import os
from utah.provisioning.inventory.exceptions import \
        UTAHProvisioningInventoryException
from utah.provisioning.inventory.sqlite import SQLiteInventory
from utah.provisioning.baremetal.cobbler import CobblerMachine


class BootspeedInventory(SQLiteInventory):
    """
    Keep an inventory of manually entered machines for bootspeed testing.
    All columns other than machineid, name, and state are assumed to be
    arguments for cobbler system creation.
    """
    def __init__(self, db='~/.utah-bootspeed-inventory',
                 lockfile='~/.utah-bootspeed-lock', *args, **kw):
        db = os.path.expanduser(db)
        lockfile = os.path.expanduser(lockfile)
        if not os.path.isfile(db):
            raise UTAHProvisioningInventoryException(
                    'No machine database found at ' + db)
        super(BootspeedInventory, self).__init__(*args, db=db,
                                                 lockfile=lockfile, **kw)
        machines_count = (self.connection
                          .execute('SELECT COUNT(*) FROM machines')
                          .fetchall()[0][0])
        if machines_count == 0:
            raise UTAHProvisioningInventoryException('No machines in database')
        self.machines = []

    def request(self, name=None, *args, **kw):
        query = 'SELECT * FROM machines'
        queryvars = []
        if name is not None:
            query += ' WHERE name=?'
            queryvars.append(name)
        result = self.connection.execute(query, queryvars).fetchall()
        if result is None:
            raise UTAHProvisioningInventoryException(
                    'No machines meet criteria')
        else:
            for machineinfo in result:
                cargs = dict(machineinfo)
                machineid = cargs.pop('machineid')
                name = cargs.pop('name')
                state = cargs.pop('state')
                if state == 'available':
                    update = self.connection.execute(
                            "UPDATE machines SET state='provisioned' "
                            "WHERE machineid=? AND state='available'",
                            [machineid]).rowcount
                    if update == 1:
                        machine = CobblerMachine(*args, cargs=cargs,
                                inventory=self, name=name, **kw)
                        self.machines.append(machine)
                        return machine
                    elif update == 0:
                        raise UTAHProvisioningInventoryException(
                                'Machine was requested by another process '
                                'before we could request it')
                    elif update > 1:
                        raise UTAHProvisioningInventoryException(
                                'Multiple machines exist '
                                'matching those criteria; '
                                'database ' + self.db + ' may be corrupt')
                    else:
                        raise UTAHProvisioningInventoryException(
                                'Negative rowcount returned '
                                'when attempting to request machine')
        raise UTAHProvisioningInventoryException(
                'All machines meeting criteria are currently unavailable')

    def release(self, machine=None, name=None):
        if machine is not None:
            name = machine.name
        if name is None:
            raise UTAHProvisioningInventoryException(
                    'name required to release a machine')
        query = "UPDATE machines SET state='available' WHERE name=?"
        queryvars = [name]
        update = self.connection.execute(query, queryvars).rowcount
        if update == 1:
            if machine is not None:
                if machine in self.machines:
                    self.machines.remove(machine)
        elif update == 0:
            raise UTAHProvisioningInventoryException(
                    'SERIOUS ERROR: Another process released this machine '
                    'before we could, which means two processes provisioned '
                    'the same machine simultaneously')
        elif update > 1:
            raise UTAHProvisioningInventoryException(
                    'Multiple machines exist matching those criteria; '
                    'database ' + self.db + ' may be corrupt')
        else:
            raise UTAHProvisioningInventoryException(
                    'Negative rowcount returned '
                    'when attempting to request machine')

# Here is how I currently create the database:
# CREATE TABLE machines (machineid INTEGER PRIMARY KEY,
#                        name TEXT NOT NULL UNIQUE,
#                        state TEXT default 'available',
#                        [mac-address] TEXT NOT NULL UNIQUE,
#                        [power-address] TEXT DEFAULT '10.97.0.13',
#                        [power-id] TEXT,
#                        [power-user] TEXT DEFAULT 'ubuntu',
#                        [power-pass] TEXT DEFAULT 'ubuntu',
#                        [power-type] DEFAULT 'sentryswitch_cdu');

# Here is how I currently populate the database:
# INSERT INTO machines (name, [mac-address], [power-id])
# VALUES ('acer-veriton-01-Pete', 'd0:27:88:9f:73:ce', 'Veriton_1');
# INSERT INTO machines (name, [mac-address], [power-id])
# VALUES ('acer-veriton-02-Pete', 'd0:27:88:9b:84:5b', 'Veriton_2');
